package UI.GameState;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;


public class WallState {
    private int frameX, frameY;

    private String source;
    private String[] lines;
    private int numRows, numColumns, w, h;
    private final int wallWidth = 5;
    private ArrayList<WallData> walls = new ArrayList<>();
    private Image wall1 ;
    private Image wall2 ;
    private Image blank ;


    public WallState(int GAME_WIDTH, int GAME_HEIGHT) {
        frameX = GAME_WIDTH;
        frameY = GAME_HEIGHT;

        try {
            source = new String(Files.readAllBytes(Paths.get("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map3.txt")), StandardCharsets.UTF_8);
            wall1 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall1.png"));
            wall2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall2.png"));
            blank = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\blank.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        lines = source.split("\r\n|\r|\n");

        numRows = lines.length;  //number of rows
        numColumns = lines[0].trim().length();  //number of columns

        h = (frameY-30) / (numRows);   //sideY
        w = (frameX-10) / (numColumns);  //sideX

    }


    public void paintComponent(Graphics g) {

        Image wall1scaled = wall1.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        Image wall2scaled = wall2.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        Image blankScaled = blank.getScaledInstance(w, h, java.awt.Image.SCALE_SMOOTH);
        // Draw background
        walls.clear();
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, frameX, frameY);
        g.setColor(Color.black);  //colour of the walls
        int x = 0;
        int y = 0;

        for (int r = 0; r < numRows; r++) {
            x = 0 ;
            String rowChars = lines[r];
            for (int c = 0; c < numColumns; c++) {
                if (rowChars.charAt(c) == '0') {
                    drawBlanks(g, x, y ,blankScaled);
                    x += w;
                }
                else if (rowChars.charAt(c) == '1') {
                    drawAndSaveWalls(g, x, y, r , c, wall1scaled);
                    x += w;
                }
                else if (rowChars.charAt(c) == '2') {
                    drawAndSaveWalls(g, x, y, r , c, wall2scaled);
                    x += w;
                }
            }
            y += h;
        }





    /*    int start = 10;
        int startTop = 30;
        for (int r = 0; r < numRows; r++) {
            String rowChars = lines[r];
            for (int c = 0; c < numColumns; c++) {
                if (rowChars.charAt(c)=='0') continue;

                if (c==0 && r==0 ){
                    if (rowChars.charAt(c+1)=='1') draw(g,start,startTop,w+(w/2),wallWidth);
                    if (lines[r+1].charAt(c)=='1') draw(g,start,start,wallWidth,h);
                    continue;
                }

                if (r==0){
                    if (lines[r+1].charAt(c)=='1'){
                        if (c==numColumns-1) {
                            draw(g,frameX-start-wallWidth,startTop,wallWidth,h);
                        }else{
                            draw(g,(c*w)+(w/2)+start,startTop,wallWidth,h);
                        }
                    }
                }

                if ((c<numColumns-1 && rowChars.charAt(c+1)=='1') || (c>0 && rowChars.charAt(c-1)=='1')){ //horizontal
                    int y = (r*h) + startTop;
                    int x = start;
                    if (r == numRows -1)
                        y = frameY - start - wallWidth;//last row
                    if (r<numRows-1 && r>0)
                        y += h/4;
                    if (c>0)
                        x = (w*c)+(w/2)+start;
                    if (r==numRows-1 && c>0)
                        x -=w/2;
                    draw(g,x,y,w,wallWidth);
                }

                if ((r>0 && lines[r-1].charAt(c)=='1') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='1')) { //vertical
                    int x = start;
                    int y = r * h +start;
                    int th = h;
                    if (c>0)
                        x = (c*w) + (w/2) + start;
                    if (r==numRows-1)
                        th += startTop*2;
                    if (c==numColumns-1)//last column
                        x = frameX - start - wallWidth;
                    draw(g,x, y, wallWidth, th);
                }

            }
        }

        */
    }



    private void drawAndSaveWalls(Graphics g2d,int x,int y , int r , int c ,Image image){
        WallData wall = new WallData(x , y , w, h);
        if ((c<numColumns-1 && lines[r].charAt(c+1)=='1') || (c>0 && lines[r].charAt(c-1)=='1')){ //horizontal
            wall.setHorizontal(true);
        }
        if ((r>0 && lines[r-1].charAt(c)=='1') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='1')) { //vertical
            wall.setVertical(true);
        }
        walls.add(wall);
        g2d.drawImage(image , x , y , null);
    }

    private void drawBlanks(Graphics g2d,int x,int y,Image image){
        g2d.drawImage(image , x , y , null);
    }

    public boolean isWall(int x,int y){
        for(WallData d : walls){
            if (d.isWall(x,y)) return true;
        }
        return false;
    }


    public ArrayList<WallData> getWalls() {
        return walls;
    }

    public class WallData {
        private int x,y,w,h;
        private boolean horizontal = false;
        private boolean vertical = false;

        public WallData(int x,int y,int w,int h){
            this.x = x;
            this.y = y;
            this.h = h;
            this.w = w;
        }

        public boolean isWall (int x,int y){
            //BUG
            return (x>this.x && x<this.x+w && y>this.y && y<this.y+h);
        }

        public boolean isHorizontal() {
            return horizontal;
        }

        public void setHorizontal(boolean horizontal) {
            this.horizontal = horizontal;
        }

        public boolean isVertical() {
            return vertical;
        }

        public void setVertical(boolean vertical) {
            this.vertical = vertical;
        }
    }
}
